@@ -12,13 +12,13 @@ from datetime import datetime
 import numpy as np
 from pathlib import Path
 import re
-import json
 from textwrap import wrap
+import json
 from math import pi
 import sys
 
 ################
-VERSION="1.0.0"
+VERSION="1.0.1"
 ################
 
 def insertCopyNumber(outfilename, filename):
@@ -38,7 +38,7 @@ def formatting(text): #Thank you to user blackpen on StackExchange for sharing t
 
 def getArgsFromCommandLine():
     indent_formatter = lambda prog: argparse.RawTextHelpFormatter(prog,max_help_position=40)
-    parser = argparse.ArgumentParser(description='Astronomy package written in Python.\n\nRequired options are:\n\n-filename\n-scale\n-ra\n-dec',add_help=False,formatter_class=indent_formatter)
+    parser = argparse.ArgumentParser(description='Astrometry package written in Python.\n\nRequired options are:\n\n-filename\n-scale\n-ra\n-dec',add_help=False,formatter_class=indent_formatter)
     parser.add_argument('-h', '-help', '--h', '--help', action='help', default=argparse.SUPPRESS, help=formatting('Shows this help message.'))
     parser.add_argument('-filename', type=Path, help=formatting('This is the full file path + name of a FITS image file to solve the WCS solution for.'))
     parser.add_argument('-ra', help=formatting('The approximate right-ascension of the field center of the image. This can be supplied in either right-ascension sexagesimal (HH:MM:SS.S) format, degree.decimal format, or as the keyword in the FITS file which contains the relevant value in either right-ascension sexagesimal or degree.decimal format.'))
@@ -52,7 +52,7 @@ def getArgsFromCommandLine():
     parser.add_argument('-catalogue', help=formatting('This specifies the catalogue to query for sources in the given nominal image field. The default is Gaia DR3, in its \'g\' band. To specify other filter bands, see the -filter option setting. Possible catalogues options are: \n\n \'GaiaDR3\'.'))
     parser.add_argument('-filter', help=formatting('This specifies the filter to use from a given catalogue. Default catalogue and filter are Gaia DR3 in \'g\' band. If a specified filter isn\'t found in a given catalogue (specified or default), an error is thrown. Valid filters for given catalogues are as follows: \n\nGaia DR3: \'bp\' (blue), \'g\' (green), \'rp\' (red).'))
     parser.add_argument('-pmepoch', help=formatting('Pass the year.year value of the observation to update the RA and Dec entries of the table with their proper motion adjustments, given the catalogue reference epoch. Only entries in the query which have valid proper motion entries will be saved to ouput.'))
-    parser.add_argument('-npts', help=formatting('The number of sources to use in both the catalogue and image source extraction. Suggest 25 for good wavelength correspondence between the catalogue and image; 50 for poor correspondence; 100 for very poor; 200 for a hope and a prayer and you might crash your system...but it still works often. 50 is the default when not supplied.'))
+    parser.add_argument('-npts', help=formatting('The number of sources to use in both the catalogue and image source extraction. Suggest 25 for good wavelength correspondence between the catalogue and image; 50 for poor correspondence; 100 for very poor; 200 for a hope and a prayer and you might crash your system...but it still works often. 75 is the default when not supplied.'))
     parser.add_argument('-nrefinepts', help=formatting('This is the number of points to use for automatic refinement of the initial solution based on npts; npts must be a relatively smaller value (see comment on npts) given the (n-choose-3)^2 nature of the solver, but the initial solution may be easily refined with a much larger number of points. Default is 500; maximum is 1000; minimum is npts.'))
     parser.add_argument('-pixsat', help=formatting('This is important and very helpful to use if the image is known to have saturated pixels. One can inform the point source extractor the threshold above which sources will be expected to be saturated and form island-plateaus instead of Gaussian peaks. For example, a pixel-well may saturate at an ADU value of ~50,000, although it might have a 16-bit ADC which would saturate at 65,536 pixel value. In this case a \'pixsat\' should be specified as, say, 45,000, just below the ADU well-depth. If -pixsat is not supplied, there will be no special treatment of saturated sources.'))
     parser.add_argument('-kernelrad', help=formatting('This is the radius of the centroiding kernel used in the point-source extractor. Default, and minimum, is 2, which means that +-2 pixels about a source peak will be used for centroiding the source, i.e., a 5x5 pixel array centered on the source peak is centroided. If the sources are over-sampled then the user might like to set a larger radius such that the kernel samples the complete PSF. The default of 2 assumes that the image source PSFs are critically-sampled.'))
@@ -60,7 +60,7 @@ def getArgsFromCommandLine():
     parser.add_argument('-vertextol', help=formatting('Default 0.25 degrees. This sets the vertex tolerance within which two vertices of a triangle must match in angle to another triangle in order to be considered a potential pattern-match between the image source coordinates and catalogue source coordinates. Not much testing has been done on this option and 0.25 has always been used. A smaller tolerance will be less forgiving to distortion but would offer faster searching and fewer false-positives, whereas a larger tolerance will be more forgiving to distortion at the expense of slower searching and a higher rate of false-positives.'))
     parser.add_argument('-nmatchpoints', help=formatting('This specifies the number of points which must match between the point source list and catalogue list to be considered a solution. Default is 6 points. Minimum is 3 but this is very likely to produce false-positive solutions. 4 or 5 can be used where there are simply very few points in the source image. 6 or higher should guarantee no false positives.'))
     parser.add_argument('-nmatchpercent', help=formatting('Same as above but as a percentage of point sources available. Default is 25%%. If both nmatchpoints and nmatchpercent are supplied, the smaller threshold will be used, found by comparing nmatchpoints to nmatchpercent*npts.'))
-    parser.add_argument('--save', metavar='ID', help=formatting('Saves current options to a json file. The user must supply an ID consisting of arbitrary characters that will get written into the filename. This way, multiple sets of options can be saved.'))
+    parser.add_argument('--save', metavar='ID', help=formatting('Save the options settings as a file to disk to recall later. User supplies an ID, for example: "--save UVIT". Useful if frequently processing different telescopic fields, allowing quick input of WCS solver settings. Saves all settings EXCEPT for filename, ra, dec, pmepoch, so that different images but taken from the same telescope can be processed at their unique individual characteristics.'))
     parser.add_argument('--load', metavar='ID', help=formatting('Loads options from the json file whose filename contains the ID specified. Any supplied options will be overwritten by the ones in the json file.'))
     parser.add_argument('--version', '-V', action='version', version='fastrometry {}'.format(VERSION), help=formatting('Shows the current version of fastrometry.'))
     parser.add_argument('--verbosity', help=formatting('Sets the verbosity level of console messages. A value of 1 represents normal verbosity and is the default. Set to 0 for a silent run, or 2 for higher verbosity.'))
@@ -92,83 +92,96 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
     if verbosity >= 1:
         print("\nParsing arguments...")
 
-    if load is None:
-        if filename is None:
+    if filename is None:
             sys.exit('ERROR: -filename is required.')          
-        else:
+    else:
+        try:
+            filename = str(filename)
+            filepath = user_dir+'\\'+filename
+            if verbosity >= 1:
+                print("| Opening fits image...")
+            hdul = fits.open('{}'.format(filepath))
+            img = hdul[0].data.astype(np.double)
+            header = hdul[0].header
+            if verbosity >= 1:
+                print("| done")
+        except Exception as e:
+            sys.exit('ERROR: Could not open the specified FITS file. Check that you are in the directory containing your FITS files.\nORIGINAL ERROR: {}'.format(e))
+            
+    if ra is None:
+        sys.exit('ERROR: -ra is required.')          
+    else:
+        if len(ra.split(':')) == 1:
             try:
-                filename = str(filename)
-                filepath = user_dir+'\\'+filename
-                if verbosity >= 1:
-                    print("| Opening fits image...")
-                hdul = fits.open('{}'.format(filepath))
-                img = hdul[0].data.astype(np.double)
-                header = hdul[0].header
-                if verbosity >= 1:
-                    print("| done")
-            except Exception as e:
-                sys.exit('ERROR: Could not open the specified FITS file. Check that you are in the directory containing your FITS files.\nORIGINAL ERROR: {}'.format(e))
-                
-        if ra is None:
-            sys.exit('ERROR: -ra is required.')          
+                ra = float(ra)              ###decimal
+                if 0 <= ra < 360:
+                    pass
+                else:
+                    sys.exit('ERROR: ra in degree.decimal format must be between 0 and 360.')                      
+            except:                            
+                if ra in header:       ###header keywords
+                    ra = float(header['{}'.format(ra)])
+                else:
+                    sys.exit('ERROR: Could not find the specified keyword in the FITS header (assuming ra was given as a keyword).')                      
+        elif len(ra.split(':')) == 3:      ###sexagesimal
+            ra_hms = ra.split(':')
+            try:
+                hrs = int(ra_hms[0])
+                mins = int(ra_hms[1])
+                secs = float(ra_hms[2])
+                if 0 <= hrs < 24 and  0 <= mins < 60 and 0 <= secs < 60:
+                    ra = (hrs + mins/60 + secs/3600)*15
+                else:
+                    sys.exit('ERROR: ra in HH:MM:SS.S format must hrs between 0 and 24, minutes between 0 and 60, and seconds between 0 and 60)')                     
+            except:
+                sys.exit('ERROR: ra in HH:MM:SS.S format must have integer hours, integer minutes, and integer or decimal seconds.')             
         else:
-            if len(ra.split(':')) == 1:
-                try:
-                    ra = float(ra)              ###decimal
-                    if 0 <= ra < 360:
-                        pass
-                    else:
-                        sys.exit('ERROR: ra in degree.decimal format must be between 0 and 360.')                      
-                except:                            
-                    if ra in header:       ###header keywords
-                        ra = float(header['{}'.format(ra)])
-                    else:
-                        sys.exit('ERROR: Could not find the specified keyword in the FITS header (assuming ra was given as a keyword).')                      
-            elif len(ra.split(':')) == 3:      ###sexagesimal
-                ra_hms = ra.split(':')
-                try:
-                    hrs = int(ra_hms[0])
-                    mins = int(ra_hms[1])
-                    secs = float(ra_hms[2])
-                    if 0 <= hrs < 24 and  0 <= mins < 60 and 0 <= secs < 60:
-                        ra = (hrs + mins/60 + secs/3600)*15
-                    else:
-                        sys.exit('ERROR: ra in HH:MM:SS.S format must hrs between 0 and 24, minutes between 0 and 60, and seconds between 0 and 60)')                     
-                except:
-                    sys.exit('ERROR: ra in HH:MM:SS.S format must have integer hours, integer minutes, and integer or decimal seconds.')             
-            else:
-                sys.exit('ERROR: ra must be entered either in HH:MM:SS.S format (e.g. "18:44:27.891"), degree.decimal format (e.g. "78.132"), or as a FITS header keyword whose value will be retrieved (e.g. "RA_PT").')
-                
-        if dec is None:
-            sys.exit('ERROR: -dec is required.')         
+            sys.exit('ERROR: ra must be entered either in HH:MM:SS.S format (e.g. "18:44:27.891"), degree.decimal format (e.g. "78.132"), or as a FITS header keyword whose value will be retrieved (e.g. "RA_PT").')
+            
+    if dec is None:
+        sys.exit('ERROR: -dec is required.')         
+    else:
+        if len(dec.split(':')) == 1:
+            try:
+                dec = float(dec)                 ###decimal
+                if -90 <= dec < 90:
+                    pass
+                else:
+                    sys.exit('ERROR: dec in degree.decimal format must be between -90 and 90.')                     
+            except:
+                if dec in header:        ###header keywords
+                    dec = float(header['{}'.format(dec)])
+                else:
+                    sys.exit('ERROR: Could not find the specified keyword in the FITS header (assuming dec was given as a keyword).')                      
+        elif len(dec.split(':')) == 3:     ###sexagesimal
+            dec_hms = dec.split(':')
+            try:
+                degs = int(dec_hms[0])
+                amins = int(dec_hms[1])
+                asecs = float(dec_hms[2])
+                if -90 <= degs < 90 and 0 <= amins < 60 and 0 <= asecs < 60:
+                    dec = degs + amins/60 + asecs/3600
+                else:
+                    sys.exit('ERROR: dec in DD:MM:SS.S" format must have degrees between -90 and 90, arcminutes between 0 and 60, and arcseconds between 0 and 60.')                      
+            except:
+                sys.exit('ERROR: dec in DD:MM:SS.S" format must have integer degrees, integer arcminutes, and integer or decimal arcseconds.')                
         else:
-            if len(dec.split(':')) == 1:
-                try:
-                    dec = float(dec)                 ###decimal
-                    if -90 <= dec < 90:
-                        pass
-                    else:
-                        sys.exit('ERROR: dec in degree.decimal format must be between -90 and 90.')                     
-                except:
-                    if dec in header:        ###header keywords
-                        dec = float(header['{}'.format(dec)])
-                    else:
-                        sys.exit('ERROR: Could not find the specified keyword in the FITS header (assuming dec was given as a keyword).')                      
-            elif len(dec.split(':')) == 3:     ###sexagesimal
-                dec_hms = dec.split(':')
-                try:
-                    degs = int(dec_hms[0])
-                    amins = int(dec_hms[1])
-                    asecs = float(dec_hms[2])
-                    if -90 <= degs < 90 and 0 <= amins < 60 and 0 <= asecs < 60:
-                        dec = degs + amins/60 + asecs/3600
-                    else:
-                        sys.exit('ERROR: dec in DD:MM:SS.S" format must have degrees between -90 and 90, arcminutes between 0 and 60, and arcseconds between 0 and 60.')                      
-                except:
-                    sys.exit('ERROR: dec in DD:MM:SS.S" format must have integer degrees, integer arcminutes, and integer or decimal arcseconds.')                
-            else:
-                sys.exit('ERROR: dec must be entered either in DD:MM:SS.S" format (e.g. "-49:12:09.255"), degree.decimal format (e.g. "78.132"), or as a FITS header keyword whose value will be retrieved (e.g. "DEC_PT").')             
+            sys.exit('ERROR: dec must be entered either in DD:MM:SS.S" format (e.g. "-49:12:09.255"), degree.decimal format (e.g. "78.132"), or as a FITS header keyword whose value will be retrieved (e.g. "DEC_PT").')             
 
+    if pmepoch is None:
+        pmepoch = 0
+    else:
+        try:
+            pmepoch = float(pmepoch)
+            if 1950 <= pmepoch:
+                pass
+            else:
+                sys.exit("ERROR: pmepoch must be greater than or equal to 1950.")
+        except:
+            sys.exit("ERROR: pmepoch must be a number.")
+
+    if load is None:
+        
         if scale is None:
             sys.exit('ERROR: scale is required.')          
         else:
@@ -272,29 +285,15 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
         else:
             sys.exit("ERROR: catalogue must be from the following options: ['GaiaDR3'].")
 
-        if pmepoch is None:
-            pmepoch = 0
-        else:
-            try:
-                pmepoch = float(pmepoch)
-                if pmepoch >= 0:
-                    pass
-                else:
-                    sys.exit("pmepoch")
-            except:
-                sys.exit("ERROR: pmepoch must be a positive number.")
-
         if npts is None:
-            npts = 50
+            npts = 75
         else:
             try:
                 npts = int(npts)
-                if npts >= 10:
-                    if npts > 200:
-                        from tkinter import messagebox
-                        messagebox.showwarning('Warning: computer may explode.')
+                if 10 <= npts <= 300:
+                    pass
                 else:
-                    sys.exit("ERROR: npts must be greater than or equal to 10.")
+                    sys.exit("ERROR: npts must be between 10 and 300.")
             except:
                 sys.exit('ERROR: npts must be an integer.')
                 
@@ -389,9 +388,6 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
                 print("Saving args to file...")
             try:
                 contents = {
-                    'filename' : filename,
-                    'ra' : ra,
-                    'dec' : dec,
                     'scale' : scale,
                     'scalebnds' : scalebnds,
                     'rotation' : rotation,
@@ -400,7 +396,6 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
                     'fieldshape' : shape,
                     'catalogue' : catalogue,
                     'filter' : filter,
-                    'pmepoch' : pmepoch,
                     'npts' : npts,
                     'nrefinepts' : nrefinepts,
                     'pixsat' : pixsat,
@@ -425,14 +420,12 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
                 print(e)
                 sys.exit("ERROR: Could not save options.")          
     else:
+        id = load
         try:
             if verbosity >= 1:
-                print("Loading arguments from saveoptions_{}.json...".format(id))
+                print("| Loading arguments from saveoptions_{}.json...".format(id))
             with open('{}\\cmd_args\\saveoptions_{}.json'.format(user_dir,id),'r') as f:
                 jobject = json.load(f)
-                filename = jobject['filename']
-                ra = jobject['ra']
-                dec = jobject['dec']
                 scale = jobject['scale']
                 scalebnds = jobject['scalebnds']
                 rotation = jobject['rotation']
@@ -441,7 +434,6 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
                 shape = jobject['fieldshape']
                 catalogue = jobject['catalogue']
                 filter = jobject['filter']
-                pmepoch = jobject['pmepoch']
                 npts = jobject['npts']
                 nrefinepts = jobject['nrefinepts']
                 pixsat = jobject['pixsat']
@@ -456,7 +448,7 @@ def find_WCS(filename=None, ra=None, dec=None, scale=None, scalebnds=None, rotat
                 img = hdul[0].data.astype(np.double)
                 header = hdul[0].header
                 if verbosity >= 1:
-                    print("done")
+                    print("| done")
         except Exception as e:
             print(e)
             sys.exit("ERROR: Could not load options. Check that a json file exists with the specified id.")    
